""" Server-package

    Initialize app, cors, database and bcrypt (for passwordhashing) and added it to the application.
    Initialize also a singelton for the AccesTokenControler and start the Thread.

"""
from pathlib import Path
_modpath = Path(__file__).parent

from flask import Flask
from flask_cors import CORS
from flask_ldapconn import LDAPConn
import ssl
import pkg_resources, yaml

from logging.config import dictConfig

with (_modpath/'logging.yml').open(mode='rb') as file:
    config = yaml.safe_load(file.read())
    dictConfig(config)

import logging
from werkzeug.local import LocalProxy
logger = LocalProxy(lambda: logging.getLogger(__name__))


def create_app():
    app = Flask(__name__)
    CORS(app)

    with app.app_context():
        from .system.database import db
        from .system.config import configure_app, config
        configure_app(app)
        db.init_app(app)

        for entry_point in pkg_resources.iter_entry_points('flaschengeist.auth'):
            logger.debug('Found authentification plugin: %s', entry_point.name)
            if entry_point.name == config['FLASCHENGEIST']['AUTH']:
                app.config['FG_AUTH_BACKEND'] = entry_point.load()()
                app.config['FG_AUTH_BACKEND'].configure(config[entry_point.name] if config.has_section(entry_point.name) else None)
                logger.info('Loaded authentification plugin > %s <', entry_point.name)
                break
        if not app.config['FG_AUTH_BACKEND']:
            logger.error('No authentification plugin configured or authentification plugin not found')

    logger.info('Search for plugins')
    discovered_plugins = {
        entry_point.name: entry_point.load()
        for entry_point in pkg_resources.iter_entry_points('flaschengeist.plugin')
    }

    for name in discovered_plugins:
        logger.debug("Found plugin: %s", name)
        if config.get(name, 'enabled', fallback=False):
            logger.info('Loaded plugin > %s <', name)
            app.register_blueprint(discovered_plugins[name]())

    return app
#app.register_blueprint(baruser)
#app.register_blueprint(finanzer)
#app.register_blueprint(user)
#app.register_blueprint(vorstand)
#app.register_blueprint(gastrouser)
#app.register_blueprint(registration)
